/*=============================================================================
	Fragmentshader for DrawComplex, multi pass, LightMap.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

uniform sampler2D Texture0;
uniform vec4 DrawColor;
uniform uint PolyFlags;
uniform float Gamma;

in vec2 vFogMapCoords;

out vec4 FragColor;

void main (void)
{
	vec4 TotalColor = texture(Texture0, vFogMapCoords)*2.0; // For FogMap

	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked && TotalColor.a < 0.5 )
	{
		TotalColor.rgb /= TotalColor.a;
		discard;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && TotalColor.a < 0.01 )
		discard;

	if (DrawColor != 0.0)
		TotalColor *= DrawColor; // Add DrawColor.

	// Gamma
	TotalColor.r=pow(TotalColor.r,2.0-Gamma);
	TotalColor.g=pow(TotalColor.g,2.0-Gamma);
	TotalColor.b=pow(TotalColor.b,2.0-Gamma);

	FragColor = TotalColor;
}
