/*=============================================================================
	Fragmentshader for DrawComplexSurface, multi pass, MacroTextures.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

uniform sampler2D Texture0;
uniform uint PolyFlags;
uniform float Gamma;

uniform vec4 DrawColor;
uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));

	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);

	return FogResult;
}

in vec2 vMacroTexCoords;
in vec4 vEyeSpacePos;

out vec4 FragColor;

void main (void)
{
	vec4 TotalColor = texture(Texture0, vMacroTexCoords); // For MacroTextures
	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked && TotalColor.a < 0.5 )
	{
		TotalColor.rgb /= TotalColor.a;
		discard;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && TotalColor.a < 0.01 )
		discard;

	if (DrawColor != 0.0)
		TotalColor *= DrawColor; // Add DrawColor.

	// Add distance fog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		vec4 FogColor = FogParams.FogColor;
		if ( (PolyFlags&PF_Modulated) == PF_Modulated )
			FogColor = vec4(0.5,0.5,0.5,0.0);
		else if ( (PolyFlags&PF_Translucent) == PF_Translucent )
			FogColor = vec4(0.0,0.0,0.0,0.0);

		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogColor, getFogFactor(FogParams, FogCoord));
	}

	// Gamma
	TotalColor.r=pow(TotalColor.r,1.8-Gamma);
	TotalColor.g=pow(TotalColor.g,1.8-Gamma);
	TotalColor.b=pow(TotalColor.b,1.8-Gamma);

	FragColor = TotalColor;
}
