/*=============================================================================
	Vertexshader for DrawComplexSurface, single pass.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

layout (location = 0) in vec3 Coords;		// == gl_Vertex
layout (location = 1) in vec3 Normals;		// Surface Normals

layout(std140) uniform GlobalMatrices
{
	mat4 modelviewprojMat;
	mat4 modelviewMat;
};

uniform vec4 TexCoords[10];
uniform vec4 LightPos[8];

out vec3 vCoords;
out vec3 vNormals;
out vec4 vEyeSpacePos;
out vec2 vTexCoords;
out vec2 vLightMapCoords;
out vec2 vFogMapCoords;
out vec2 vDetailTexCoords;
out vec2 vMacroTexCoords;
out vec2 vBumpTexCoords;
out vec2 vEnvironmentTexCoords;
out vec4 vLightPos[8];

void main(void)
{
	vec4 vEyeSpacePosVertex = modelviewMat*vec4(Coords, 1.0);
	vEyeSpacePos = vEyeSpacePosVertex;

	// Point Coords
	vCoords = Coords;


	// UDot/VDot calculation.
	vec3 MapCoordsXAxis = TexCoords[7].xyz;
	vec3 MapCoordsYAxis = TexCoords[8].xyz;
	vec3 MapCoordsOrigin = TexCoords[9].xyz;

	float UDot = dot(MapCoordsXAxis,MapCoordsOrigin);
	float VDot = dot(MapCoordsYAxis,MapCoordsOrigin);
	float MapDotU = dot(MapCoordsXAxis,Coords) - UDot;
	float MapDotV = dot(MapCoordsYAxis,Coords) - VDot;
	vec2  MapDot  = vec2(MapDotU,MapDotV);

	vec2 TexMapMult = TexCoords[0].xy;
	vec2 TexMapPan = TexCoords[0].zw;

	vec2 LightMapMult = TexCoords[1].xy;
	vec2 LightMapPan = TexCoords[1].zw;

	vec2 DetailTexMapMult = TexCoords[2].xy;
	vec2 DetailTexMapPan = TexCoords[2].zw;

	vec2 MacroTexMapMult = TexCoords[3].xy;
	vec2 MacroTexMapPan = TexCoords[3].zw;

	vec2 BumpMapTexMapMult = TexCoords[4].xy;
	vec2 BumpMapTexMapPan = TexCoords[4].zw;

	vec2 FogMapTexMapMult = TexCoords[5].xy;
	vec2 FogMapTexMapPan = TexCoords[5].zw;

	vec2 EnvironmentMapTexMapMult = TexCoords[6].xy;
	vec2 EnvironmentMapTexMapPan = TexCoords[6].zw;

	//Texture UV to fragment
	vTexCoords = (MapDot - TexMapPan) * TexMapMult;

	//Texture UV Lightmap to fragment
	vLightMapCoords	= (MapDot - LightMapPan) * LightMapMult;

	// Texture UV DetailTexture
	vDetailTexCoords = (MapDot - DetailTexMapPan) * DetailTexMapMult;

	// Texture UV Macrotexture
	vMacroTexCoords = (MapDot - MacroTexMapPan) * MacroTexMapMult;

	// Texture UV BumpMap
	vBumpTexCoords = (MapDot - BumpMapTexMapPan) * BumpMapTexMapMult;

	// Texture UV FogMap
	vFogMapCoords = (MapDot - FogMapTexMapPan) * FogMapTexMapMult;

	// Texture UV EnvironmentMap
	vEnvironmentTexCoords = (MapDot - EnvironmentMapTexMapPan) * EnvironmentMapTexMapMult;


	for( int i=0; i<8; ++i)
	{
		//Set the lightposition
		vLightPos[i] = normalize(LightPos[i]*modelviewMat); 
	}

	// Normals
	vNormals = Normals;

	gl_Position = modelviewprojMat * vec4(Coords, 1.0);
}
