/*=============================================================================
	Fragmentshader for DrawTile.

	Revision history:
		* Created by Smirftsch
=============================================================================*/

uniform sampler2D Texture0;
uniform uint PolyFlags;
uniform bool bHitTesting;
uniform float Gamma;

in vec2 gTexCoords;
in vec4 gDrawColor;

out vec4 FragColor;

void main(void)
{
	vec4 TotalColor;
	vec4 Color = texture(Texture0, gTexCoords);

	// Handle PF_Masked.
	if ( (PolyFlags&PF_Masked) == PF_Masked )
	{
		if(Color.a < 0.5)
			discard;
		else Color.rgb /= Color.a;
	}
	else if ( (PolyFlags&PF_AlphaBlend) == PF_AlphaBlend && Color.a < 0.01 )
		discard;

	TotalColor = Color * gDrawColor; // Add DrawColor.

	if((PolyFlags & PF_Modulated)!=PF_Modulated)
	{
		// Gamma
		TotalColor.r=pow(TotalColor.r,2.0-Gamma);
		TotalColor.g=pow(TotalColor.g,2.0-Gamma);
		TotalColor.b=pow(TotalColor.b,2.0-Gamma);
	}

#ifdef EDITOR
	// Editor support.
	if ( (PolyFlags&PF_Selected) == PF_Selected )
	{
		TotalColor.r *= 0.125 + 0.1;
		TotalColor.g *= 0.125 + 0.1;
		TotalColor.b = (TotalColor.b*0.125) + 0.5;
		TotalColor = clamp(TotalColor,0.0,1.0);
	}
	
	// HitSelection, Zoneview etc.
	if (bHitTesting)
		TotalColor = gDrawColor; // Use DrawColor.
#endif

	FragColor = TotalColor;
}
